/**
Action Script library. Button class definition
Copyright (C) 2005  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#ifndef _AS_BUTTON_HH
#define _AS_BUTTON_HH
#include "as_object.hh"
#include <map>

class ASButton;
typedef void (ASButton::*funcPtr)();

typedef map<int, float *> IntInvMatrixMap;
typedef map<int, RECT> IntRectMap;
typedef stdext::hash_map<int, ASInterface*, HashInt, HashIntCompare> IntASInterfaceHashMap;

enum buttonStates {
	buttonUp	= 0,
	buttonOver	= 1,
	buttonDown	= 2,
	buttonHit 	= 3
};

class ASButton : public ASObject{
public:
	ASButton(ASButton*);
	ASButton(Script *s, Graphics *g);
	~ASButton();
	Function *getFunctionMember(wstring memberName);
	void 	callFunction(Function *f);
	void 	init();
	ASInterface*getNewInstance(Script *s,Graphics *g) { return new ASButton(s,g); }
	ValueObject*getMember(wstring memberName);

	void	getDepth();
	void	setButtton(DefineButton2Tag *t);
	void	registerSound(DefineButtonSoundTag *);
	void	toGraphic();
	RECT* 	getSelectionBox() { 		 // Overloaded since selection area is different from ShapeArea
		puts("Don't use getSelectionBox for buttons!."); 
		return getBounds(); 
	}
	void 	callEvent(string eventName);
	list<BUTTONRECORD*>	*getActiveCharacters() { return &activeCharacters; }
	bool 	isInSelectionBox(int x, int y);
	void 	setInverseMatrix(float *,int p=0);
	ASInterface* getObjectDeepCopy(bool deep);
	bool	mustDraw(BUTTONRECORD* b);

	// CallBacks
	void	onDragOut();
	void	onDragOver();
	void 	onKeyUp();
	void	onKillFocus();
	void	onPress();
	void	onRelease();
	void	onReleaseOutside();
	void	onRollOut();
	void	onRollOver();
	void	onSetFocus();

/* Properties
	char	_alpha;
	char	enabled;
	char	_focusrect;
	char	_height;
	char	_highquality;
	char	menu;
	char	_name;
	char	_parent;
	char	_quality;
	char	_rotation;
	char	_soundbuftime;
	char	tabEnabled;
	char	tabIndex;
	char	_target;
	char	trackAsMenu;
	char	_url;
	char	useHandCursor;*/
	bool	handleThisKey(int key);
	void	computeBounds();
private:
	DefineButton2Tag 	*tag;
	DefineButtonSoundTag*sound;
	int					buttonState;
	long				__zzbug;
	IntRectMap			*hitBounds;
	long				__zzbug2;
	list<BUTTONRECORD*> activeCharacters;
	IntInvMatrixMap 	invMatrixes;
	wstring				selectedMovie;
	IntASInterfaceHashMap shapes;
	float	v0,vf;
	

	funcPtr	eventDispatcher(string eventName);
	void 	processActions(int state);
	bool 	callInternalEvent(string eventName, void *data);
	bool	checkNeedRedraw();
	float	getWidth();
	float	getHeight();
	ASInterface *getMovieFromButtonState(int buttonState);
};
#endif
