/**
Action Script library. Function class imlplementation
Copyright (C) 2005

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#include "as_function.hh"
#include "../lib/event_manager.hh"

typedef void (ASFunction::*funcPtrFunction)();
static funcPtrFunction callbacks[]= { 
	&ASFunction::init,			// 0
	&ASFunction::apply,			// 1
	&ASFunction::call,			// 2
	&ASFunction::clearInterval,	// 3
	&ASFunction::eval,			// 4
	&ASFunction::fscommand,		// 5
	&ASFunction::gotoAndPlay,	// 6
	&ASFunction::gotoAndStop,	// 7
	&ASFunction::isNaN,			// 8
	&ASFunction::loadMovieNum,	// 9
	&ASFunction::loadVariablesNum,// 10
	&ASFunction::MMExecute,		// 11
	&ASFunction::nextScene,		// 12
	&ASFunction::on,			// 13
	&ASFunction::onClipEvent,	// 14
	&ASFunction::parseFloat,	// 15
	&ASFunction::parseInt,		// 16
	&ASFunction::print,			// 17
	&ASFunction::printAsBitmap,	// 18
	&ASFunction::printAsBitmapNum,// 19
	&ASFunction::printNum,		// 20
	&ASFunction::setInterval,	// 21
	&ASFunction::setProperty,	// 22
	&ASFunction::stopAllSounds,	// 23
	&ASFunction::trace,			// 24
	&ASFunction::updateAfterEvent,// 25

	// Need to be at the end.
	&ASFunction::addProperty,		
	&ASFunction::registerClass,	
	&ASFunction::AStoString,		
	&ASFunction::unwatch,			
	&ASFunction::valueOf,			
	&ASFunction::watch			
};

/**
 * The constructor
 **/
ASFunction::ASFunction(Script *s, Graphics *g) : ASObject(s,g,L"Function") {
}

/**
 * The destructor
 **/
ASFunction::~ASFunction() {
}

/**
 * Return a function descriptor
 */
Function *ASFunction::getFunctionMember(wstring memberName) {
	Function *func = getBaseFunction(memberName);
	if (memberName == L"Function") 		{func->setFunctionIndex(0); func->setNumParams(0); }
	if (memberName == L"apply") 		{func->setFunctionIndex(1); func->setNumParams(2); }
	if (memberName == L"call") 			{func->setFunctionIndex(2); func->setNumParams(10); } // inifinate
	if (memberName == L"clearInterval") {func->setFunctionIndex(3); func->setNumParams(1); }
	if (memberName == L"eval") 			{func->setFunctionIndex(4); func->setNumParams(1); }
	if (memberName == L"fscommand") 	{func->setFunctionIndex(5); func->setNumParams(2); }
	if (memberName == L"gotoAndPlay") 	{func->setFunctionIndex(6); func->setNumParams(2); }
	if (memberName == L"gotoAndStop") 	{func->setFunctionIndex(7); func->setNumParams(2); }
	if (memberName == L"isNaN") 		{func->setFunctionIndex(8); func->setNumParams(1); }
	if (memberName == L"loadMovieNum") 	{func->setFunctionIndex(9); func->setNumParams(3); }
	if (memberName == L"loadVariablesNum"){func->setFunctionIndex(10); func->setNumParams(3); }
	if (memberName == L"MMExecute") 	{func->setFunctionIndex(11); func->setNumParams(1); }
	if (memberName == L"nextScene") 	{func->setFunctionIndex(12); func->setNumParams(0); }
	if (memberName == L"on") 			{func->setFunctionIndex(13); func->setNumParams(1); }
	if (memberName == L"onClipEvent") 	{func->setFunctionIndex(14); func->setNumParams(1); }
	if (memberName == L"parseFloat") 	{func->setFunctionIndex(15); func->setNumParams(1); }
	if (memberName == L"parseInt") 		{func->setFunctionIndex(16); func->setNumParams(2); }
	if (memberName == L"print") 		{func->setFunctionIndex(17); func->setNumParams(2); }
	if (memberName == L"printAsBitmap") {func->setFunctionIndex(18); func->setNumParams(2); }
	if (memberName == L"printAsBitmapNum"){func->setFunctionIndex(19); func->setNumParams(2); }
	if (memberName == L"printNum") 		{func->setFunctionIndex(20); func->setNumParams(2); }
	if (memberName == L"setInterval") 	{func->setFunctionIndex(21); func->setNumParams(10); } // infinite
	if (memberName == L"setProperty") 	{func->setFunctionIndex(22); func->setNumParams(3); }
	if (memberName == L"stopAllSounds") {func->setFunctionIndex(23); func->setNumParams(0); }
	if (memberName == L"trace") 		{func->setFunctionIndex(24); func->setNumParams(1); }
	if (memberName == L"updateAfterEvent"){func->setFunctionIndex(25); func->setNumParams(0); }
	func= finishFunctionSetup(memberName,26,func);
	return func;
}

/**
 * Call class built in functions
 **/
void ASFunction::callFunction(Function *func) {
	numPassedParams = func->getNumPassedParams();
	(this->*callbacks[func->getFunctionIndex()])();
}

/**
 *
 **/
void ASFunction::init() {
	puts("TODO");
}

/**
 * Enables ActionScript code to call a function.
 **/
void ASFunction::apply() {
	puts("TODO");
	noReturnValue();
}

/**
 * Invokes the function represented by a Function object.
 **/
void ASFunction::call() {
	puts("TODO");
	noReturnValue();
}

/**
 *
 **/
void ASFunction::setInterval() {
	if (numPassedParams > 1) {
		wstring functionName = script->getRegister(1)->toString();
		int interval = int(script->getRegister(2)->toNumeric());
		list<ValueObject*> l;
		for (int i=3; i< numPassedParams; i++) l.push_back(script->getRegister(i)->clone());
		CallBackRequest *c = new CallBackRequest(getObjectStore(),functionName,interval,l);
		script->getEventManager()->addTimerCallbackEntry(c);
	}
	noReturnValue();
}

/**
 * Convert String to Integer
 *
 * Would have been cleaner to call a toNumeric(), but does not handle base 16 string 
 * The version with 2 parameters, need this processing.
 **/
void ASFunction::parseInt() {
	char ascii[100], *tmp;
	ValueObject *res = NULL;
	if (numPassedParams == 1) {
		wstringToUtf8(script->getRegister(1)->toString(),ascii);		// Should be ASCII
		int c;
		if (ascii[0] == '0' &&  ascii[1] == 'x') c=strtol(ascii,&tmp,16); else c= strtol(ascii,&tmp,10);
		if (*tmp == 0) {
			res = new IntegerObject(c);
		} else if (script->getRegister(1)->toString() == L"false") {
			res = new IntegerObject(0);			
		} else if (script->getRegister(1)->toString() == L"true") {
			res = new IntegerObject(1);
		} else {
			res = new StringObject(L"NaN");
		}
	} else if (numPassedParams == 2) {
		int base = int(script->getRegister(2)->toNumeric());
		wstringToUtf8(script->getRegister(1)->toString(),ascii);		// Should be ASCII
		int c;
		c=strtol(ascii,&tmp,base);
		if (*tmp == 0) {
			res = new IntegerObject(c);
		} else if (script->getRegister(1)->toString() == L"false") {
			res = new IntegerObject(0);			
		} else if (script->getRegister(1)->toString() == L"true") {
			res = new IntegerObject(1);
		} else {
			res = new StringObject(L"NaN");
		}
	} else {
		res = new ValueObject();
	}
	script->push(res);
}

void ASFunction::clearInterval() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::eval() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::fscommand() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::gotoAndPlay() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::gotoAndStop() {
	puts("TODO");
	noReturnValue();
}

/**
 * Evaluates the parameter and returns true if the value is not a number (NaN), indicating the presence of mathematical errors.
 **/
void ASFunction::isNaN() {
	bool res=true;
	if (numPassedParams > 0) {
		ValueObject *v=script->getRegister(1);
		if (v->getThisObjectInstanceName() == L"_$$$Number" || v->getThisObjectInstanceName() == L"Number") {
			res = false;
		} else if (v->getThisObjectInstanceName() == L"_$$$Boolean") {
			res = false;
		} else {
			float a;
			int i = swscanf(v->toString().c_str(),L"%f",&a);
			if (i>0) res = false;
		}
	} 
	script->push(new BooleanObject(res));
}

void ASFunction::loadMovieNum() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::loadVariablesNum() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::MMExecute() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::nextScene() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::on() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::onClipEvent() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::parseFloat() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::print() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::printAsBitmap() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::printAsBitmapNum() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::printNum() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::setProperty() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::stopAllSounds() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::trace() {
	puts("TODO");
	noReturnValue();
}
void ASFunction::updateAfterEvent() {
	puts("TODO");
	noReturnValue();
}

