/**
Action Script library. XML class implementation
Copyright (C) 2005 Christophe Choquet (cchoquet@users.sourceforge.net) 

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#include "as_xml.hh"
#include "as_xml_node.hh"

typedef void (ASXml::*funcPtr)();
static funcPtr callbacks[]= { 
	&ASXml::init,				// 0
	&ASXml::addRequestHeader,	// 1
	&ASXml::appendChild,		// 2
	&ASXml::cloneNode,			// 3
	&ASXml::createElement,		// 4
	&ASXml::createTextNode,		// 5
	&ASXml::getBytesLoaded,		// 6
	&ASXml::getBytesTotal,		// 7
	&ASXml::hasChildNodes,		// 8
	&ASXml::insertBefore,		// 9
	&ASXml::load,				// 10
	&ASXml::parseXML,			// 11
	&ASXml::removeNode,			// 12
	&ASXml::send,				// 13
	&ASXml::sendAndLoad,		// 14
	&ASXml::onData,				// 15
	&ASXml::onLoad,				// 16
	// Need to be at the end.
	&ASXml::addProperty,		
	&ASXml::registerClass,	
	&ASXml::AStoString,		
	&ASXml::unwatch,			
	&ASXml::valueOf,			
	&ASXml::watch
};

/**
 * The constructor
 **/
ASXml::ASXml(Script *s, Graphics *g) : ASObject(s,g,L"XML") {
	source = NULL;
}

/**
 * The destructor
 **/
ASXml::~ASXml(){
}

/**
 * Return a function descriptor
 */
Function *ASXml::getFunctionMember(wstring memberName) {
	Function *func = getBaseFunction(memberName);
	if (memberName == L"XML") 			{func->setFunctionIndex(0); func->setNumParams(1); }
	if (memberName == L"addRequestHeader"){func->setFunctionIndex(1); func->setNumParams(2); }
	if (memberName == L"appendChild") 	{func->setFunctionIndex(2); func->setNumParams(1); }
	if (memberName == L"cloneNode") 	{func->setFunctionIndex(3); func->setNumParams(1); }
	if (memberName == L"createElement")	{func->setFunctionIndex(4); func->setNumParams(1); }
	if (memberName == L"createTextNode") {func->setFunctionIndex(5); func->setNumParams(1); }
	if (memberName == L"getBytesLoaded") {func->setFunctionIndex(6); func->setNumParams(0); }
	if (memberName == L"getBytesTotal")	{func->setFunctionIndex(7); func->setNumParams(0); }
	if (memberName == L"hasChildNodes")	{func->setFunctionIndex(8); func->setNumParams(0); }
	if (memberName == L"insertBefore") 	{func->setFunctionIndex(9); func->setNumParams(2); }
	if (memberName == L"load") 			{func->setFunctionIndex(10); func->setNumParams(1); }
	if (memberName == L"parseXML") 		{func->setFunctionIndex(11); func->setNumParams(1); }
	if (memberName == L"removeNode") 	{func->setFunctionIndex(12); func->setNumParams(0); }
	if (memberName == L"send") 			{func->setFunctionIndex(13); func->setNumParams(2); }
	if (memberName == L"sendAndLoad") 	{func->setFunctionIndex(14); func->setNumParams(2); }
	if (memberName == L"onData") 		{func->setFunctionIndex(15); func->setNumParams(1); }
	if (memberName == L"onLoad") 		{func->setFunctionIndex(16); func->setNumParams(1); }
	func = finishFunctionSetup(memberName,17,func);
	return func;
}

/**
 * Call class built in functions
 **/
void ASXml::callFunction(Function *func) {
	numPassedParams = func->getNumPassedParams();
	(this->*callbacks[func->getFunctionIndex()])();
}

/**
 * One argument may be passed to specify a source
 **/
void ASXml::init() {
	if (numPassedParams > 0) {
		convertInputSource(script->getRegister(1)->toString());
	}
}

/**
 *  Adds or changes HTTP headers for POST operations.
 **/
void ASXml::addRequestHeader() {
	puts("TODO");
	noReturnValue();
}

/**
 * Appends a node to the end of the specified object’s child list.
 **/
void ASXml::appendChild() {
	puts("TODO");
	noReturnValue();
}

/**
 * Clones the specified node and, optionally, recursively clones all children.
 **/
void ASXml::cloneNode() {
	puts("TODO");
	noReturnValue();
}

/**
 * Creates a new XML element.
 **/
void ASXml::createElement() {
	puts("TODO");
	noReturnValue();
}

/**
 * Creates a new XML text node.
 **/
void ASXml::createTextNode() {
	puts("TODO");
	noReturnValue();
}

/**
 * Returns the number of bytes loaded for the specified XML document.
 **/
void ASXml::getBytesLoaded() {
	puts("TODO");
	noReturnValue();
}

/**
 * Returns the size of the XML document, in bytes.
 **/
void ASXml::getBytesTotal() {
	puts("TODO");
	noReturnValue();
}

/**
 * Returns true if the specified node has child nodes; otherwise, returns false.
 **/
void ASXml::hasChildNodes() {
	script->push(new BooleanObject(rootElement->children ? true : false));
}

/**
 * Inserts a node in front of an existing node in the specified node's child list.
 **/
void ASXml::insertBefore() {
	puts("TODO");
	noReturnValue();
}

/**
 * Loads a document (specified by the XML object) from a URL.
 **/
void ASXml::load() {
	puts("TODO");
	noReturnValue();
}

/**
 * Parses an XML document into the specified XML object tree.
 **/
void ASXml::parseXML() {
	if (numPassedParams > 0) {
		convertInputSource(script->getRegister(1)->toString());
	}
	doc = xmlReadMemory((char*)source,sourceLen,"noname.xml","UTF-8",0);
  	if (doc == NULL) {
		isParsed = false;
    } else {
		isParsed = true;
		rootElement = xmlDocGetRootElement(doc);
		curNode = rootElement;
	}
	noReturnValue();
}

/**
 * Removes the specified node from its parent.
 **/
void ASXml::removeNode() {
	puts("TODO");
	noReturnValue();
}

/**
 * Sends the specified XML object to a URL.
 **/
void ASXml::send() {
	puts("TODO");
	noReturnValue();
}

/**
 *  Sends the specified XML object to a URL and loads the server response into another XML object.
 **/
void ASXml::sendAndLoad() {
	puts("TODO");
	noReturnValue();
}

/**
 * Display current node value
 **/
void ASXml::AStoString() {
	if (isParsed ) {
		xmlBufferPtr buffer = xmlBufferCreate();
		if (buffer != NULL) {
			xmlNodeDump(buffer,doc,rootElement,100,0);
			wstring s = stringTowstring((const char*)buffer->content);
			xmlBufferFree(buffer);
			script->push(new StringObject(s));
		} else {
			noReturnValue();
		}
	} else {
		script->push(new StringObject(L"undefined"));
	}
}

/**
 * We don't use default member behavior for arrays
 **/
void ASXml::setMember(wstring memberName, ValueObject *v) {
}

/**
 * We don't use default member behavior for arrays
 **/
ValueObject* ASXml::getMember(wstring memberName) {
	ValueObject* res = NULL;
	if (memberName == L"firstChild") {
		res = new ASXmlNode(script,graphics,curNode,doc);
	}
	return res;
}

/**
 * Convert a wstring to a format that the XML parser can understand ie utf-8
 **/
void ASXml::convertInputSource(wstring documentText) {
	if (source != NULL) delete source;
	source = new char[documentText.size()*6];			// 6 bytes per char maximum in UTF-8
	char *ptr = source;
	sourceLen = 0;
	for (wstring::size_type i=0; i< documentText.size(); i++) {
		int j = wctomb(ptr,documentText[i]);
		ptr += j;
		sourceLen += j ;
	}
}
void  ASXml::onData() { puts("TODO"); }
void  ASXml::onLoad(){ puts("TODO"); }

