/**
Macromedia Flash File graphic lib.
Copyright (C) 2005  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#ifndef _GRAPHICS_HH_
#define _GRAPHICS_HH_
#include "common.hh"
#include "swf.hh"
#include "maths.hh"
#include "btexture.hh"
#include "object_store.hh"
#include "object_types.hh"

enum DrawingType {
	DrawingTriangles = 1,
	DrawingLines 	 = 2,
	DrawingFontChars = 3
};

class DrawingEntry{
public:
	~DrawingEntry();
	DrawingType drawingType;
	int 	texNum;
	int 	nbElements;
	int 	fillType;
	float 	lineWidth;
	RGBA	charColor;

	void 	allocateIndexes(int nbIndexes);
	unsigned int *getIndexes() { return indexes; }
	void 	allocateTexCoords(int nbCoords);
	float	*getTexCoords() { return texCoords; }
	DrawingEntry *clone();
private:
	int		nbIndexes, nbTexCoords;
	unsigned int *indexes;
	float	*texCoords;
};

// A caret for text edit
typedef struct {
	float x,y;
	float height;
	int   charOffset;
	bool  isActive;
	BaseTag *tag;
} Caret;

class DrawingSequence {
public:
	DrawingSequence();
	~DrawingSequence();
	list<DrawingEntry*> *getDrawingEntries() { return &drawingEntries;};
	void	addDrawingEntry(DrawingEntry *p) { drawingEntries.push_back(p);}
	float  	*getVertices() { return vertices; };
	void	allocateVertices(int nbVertices);
	MATRIX  *getMatrix() { return matrix; }
	void	setMatrix(MATRIX  *p) { matrix=p; }
	int		getClipDepth() { return clipDepth; }
	void	setClipDepth(int p) { clipDepth =p; }
	int		getNbPoints() { return nbPoints; }
	void	setNbPoints(int p) { nbPoints =p; }
	DrawingSequence *clone();
	void	clearEntries() { drawingEntries.clear(); }
private:
	list<DrawingEntry*> drawingEntries;
	float  	*vertices;
	MATRIX  *matrix;
	int 	clipDepth;
	int 	nbPoints, nbVertices;
};

typedef list<DrawingSequence*> ListDrawingSequence;
typedef map<unsigned int, ListDrawingSequence*> DrawingSequenceEntries;

class LocalFontManager;
class Graphics {
public:
	Graphics(BaseTexture *texture, LocalFontManager *localFontManager);
	virtual ~Graphics();
	void 	processShape(Shape *shape,int shapeId,RECT * rect);
	void	currentShapeIsCharacter();
	void	insertSequence(long characterId, DrawingSequence *d);
	void 	clearSequence(long characterId);
	ListDrawingSequence *getSequence(int characterId);
	void 	setBgColor(RGB c);
	UI8*	getRaster() { return raster; }
	void	currentShapeIsNotCharacter() { isCharacter = false; }
	void	registerStaticString(long unid,DefineTextTag* tag);
	void	registerStaticString(long unid, DefineText2Tag* tag);
	void	buildMorphedShape(int shapeId, DefineMorphShapeTag *tag, float ratio);
	void	clearRaster() { raster=NULL; }
	void	registerDynamicString(ScriptObject  *);
	Caret	getCaretLocation(DefineEditTextTag* tag, string s, float mouseX, float mouseY);
	LocalFontManager* getLocalFontManager() { return localFontManager; }
	void	registerVideoBox(long unid, int dimx, int dimy, int textureId);
	BaseTexture *getTexture() { return texture; }
	bool	cloneDrawingSequence(long unid_src, long unid_dst);
protected:
	RECT 	*rect;					// The current shape RECT
	BaseTexture *texture;
	Segment *segments;				// The current list of segments
	int 	nbSegments;
	FILLSTYLE **fillStyles;			// Pointers to fillstyles
	int		nbFillStyles;
	bool	needRefine;
	float 	*points;				// list of 2D points
	int   	nbPoints;
	int 	characterId;
private:
	bool 	isCharacter;
	UI8*	raster;					// A character texture
	DrawingSequenceEntries drawingSequences;
	bool	allowLowPrecision;
	LocalFontManager *localFontManager;
	int		defaultColorIndex;
	FILLSTYLE defaultFillStyle;


	int		getShapeAllocSize(Shape *shape);
	int		findInsertPoint(float x, float y);
	int		findInsertColor(FILLSTYLE *c);
	void	allocateShapeProcessObjects(Shape *shape);
	void	registerComputedTexture(int fillStyleType,int fillStyleIndex, FILLSTYLE *fill );
	void	updateSegments();
	void	shape2Bitmap();
	DrawingSequence *shape2Lines();
	void 	clearSequence(ListDrawingSequence *d);
	void	setMorpFillStyle(ShapeWithStyle *shape,DefineMorphShapeTag *tag, float t);
	void	setMorpLineStyle(ShapeWithStyle *shape,DefineMorphShapeTag *tag, float t);
	void	setDefaultFillStyle();

	// Implementation specific.
	virtual DrawingSequence *shape2Triangles() = 0;
};
#endif
