/**
Macromedia Flash - Event Manager definition.
Copyright (C) 2004  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#ifndef _EVENT_MANAGER_HH
#define _EVENT_MANAGER_HH
#include "common.hh"
#include "swf.hh"
#include "swf_engine.hh"
#include <iostream>
#include <stack>
#include "object_store.hh"
#include "ASLib/as_interface.hh"

typedef struct {
	ASInterface *destObject;
	wstring		destObjectTargetPath;
} INTERACTION;

typedef struct {
	ClipActions *clipActions;
	ScriptObject *so;
} EVENT_MANAGER_ENTRY;

class CallBackRequest{
public:
	CallBackRequest(ObjectStore*,wstring fn, int interval, list<ValueObject*> );
	~CallBackRequest();
	bool	requestCall();
	wstring getFunctionName() { return functionName; }
	list<ValueObject*>*getParams() { return &params; }
private:
	ObjectStore *objectStore;
	wstring functionName;
	int 	interval;
	list<ValueObject*>params;
	long	time;
	long	getTime();
};

typedef stdext::hash_map<int, INTERACTION, HashInt,  HashIntCompare> IntINTERACTIONHashMap;
typedef stdext::hash_map<string, EVENT_MANAGER_ENTRY, HashString, HashStringCompare> StringEventManagerEntryHashMap;

class EventManager {
public:
	EventManager(ObjectStore *p, Graphics *g,SwfEngine *e);
	~EventManager();
	void	registerClipAction(ScriptObject *sc,ClipActions *);
	void	unRegisterClipAction(ScriptObject *sc) { events.erase(buildKey(sc)); }
	void	doOnLoad(ScriptObject *sc);
	void	doOnLoad();
	void	doOnEnterFrame(ScriptObject *sc);
	void	doOnMouseDown(ScriptObject *sc);
	void	updateSelectionArea(int depth,ASInterface *a, int p=0);
	bool	mouseClickCallback(int x, int y,int mouseAction);
	void	keyboardCallback(int key, char *code);
	int		getMouseX() { return mouseX; }
	int		getMouseY() { return mouseY; }
	void	clearInteractions() { interactions.clear(); quitRequired = true;}
	int 	getLastKeyPressed() { return iskeyPressed ? lastKeyPressed : 0; }
	void	resetLastKeyPressed() { lastKeyPressed = 0; }
	void	cloneClipActions(ScriptObject*,ScriptObject*);
	void	addTimerCallbackEntry(CallBackRequest *c);
	void	doTimerCallbacks();
	void	setKeyPressed(bool p) { iskeyPressed = p; }
private:
	ObjectStore 		*objectStore;
	Graphics			*graphics;
	SwfEngine			*engine;
	StringEventManagerEntryHashMap events;
	IntINTERACTIONHashMap interactions;
	int					mouseX, mouseY;
	bool				quitRequired;
	int					lastKeyPressed;
	list<CallBackRequest*>timerCallbacks;
	bool				iskeyPressed;

	void	executeClipActionScript(ScriptObject *sc,ClipActionRecord *cr,bool hasLocalScope);
	void	executeFunction(ScriptObject *sc,Function *f);
	string	buildKey(ScriptObject*p) { char tmp[100]; sprintf(tmp,"%p",p); return string(tmp); }
	int		translateKeys(int keyCode);
};
#endif
