/**
Macromedia Flash File decoder dumper. 
Dumps the decoder to a flat file.
Copyright (C) 2004  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#include "common.hh"
#include "swf_decoder.hh"

/**
 *
 * Dumps the decoding output to a flat file for analysis process
 *
 *
 **/
void Swf_decoder::dumpHeader(FILE *f) {
	fprintf(f,"File type  :%c%c%c\n",header.signature1,header.signature2,header.signature3);
	fprintf(f,"Version    :%d\n",(int)header.version);
	fprintf(f,"File lenght:%d\n",header.fileLength);
	fprintf(f,"RECT	   \n");
	fprintf(f," xmin      :%d\n",(int)header.frameSize.xmin);
	fprintf(f," xmax      :%d\n",(int)header.frameSize.xmax);
	fprintf(f," ymin      :%d\n",(int)header.frameSize.ymin);
	fprintf(f," ymax      :%d\n",(int)header.frameSize.ymax);
	fprintf(f,"Frame rate :%f\n",header.frameRate);
	fprintf(f,"frame count:%d\n",header.frameCount);
	fprintf(f," 			-----------------\n");
}

void Swf_decoder::dumpRect(FILE *f, RECT *r) {
	fprintf(f," RECT:\n");
	fprintf(f,"   xmin      :%d\n",(int)r->xmin);
	fprintf(f,"   xmax      :%d\n",(int)r->xmax);
	fprintf(f,"   ymin      :%d\n",(int)r->ymin);
	fprintf(f,"   ymax      :%d\n",(int)r->ymax);

}

void Swf_decoder::dumpAction(FILE *f, DoActiontag *r) {
	fprintf(f,"** TAG DoAction\n");
	dumpActionCode(f,r->getCode());
	fprintf(f,"\n");
}

void Swf_decoder::dumpMatrix(FILE *f, MATRIX *r) {
	if (r->hasScale == 1) {
		fprintf(f,"  HasScale, scaleX=%d, scaleY=%d\n",r->scaleX,r->scaleY);
	} else {
		fprintf(f,"  noScale");
	}
	if (r->hasRotate == 1) {
		fprintf(f,"  hasRotate, rotateSkew0=%d, rotateSkew1=%d\n",r->rotateSkew0,r->rotateSkew1);
	} else {
		fprintf(f,"  noRotate");
	}
	fprintf(f,"  translateX=%d, translateY=%d\n",r->translateX,r->translateY);
}

void Swf_decoder::dumpGradient(FILE *f, GRADIENT *r) {
	if (r == NULL) return;
	list<GRADRECORD*>::iterator it;
	if (r->getGradientRecords()->begin() == NULL) return;
	for (it = r->getGradientRecords()->begin(); it != r->getGradientRecords()->end(); it++) {
		GRADRECORD *d = *it;
		fprintf(f,"  Ratio:%d  color:(%d,%d,%d,%d) \n",d->getRatio(),d->getColor()->red,d->getColor()->green,d->getColor()->blue,d->getColor()->alpha);
	}
}

void Swf_decoder::dumpMorphGradient(FILE *f, MORPHGRADIENT *r) {
	if (r == NULL) return;
	list<MORPHGRADRECORD*>::iterator it;
	if (r->getGradientRecords()->begin() == NULL) return;
	for (it = r->getGradientRecords()->begin(); it != r->getGradientRecords()->end(); it++) {
		MORPHGRADRECORD *d = *it;
		fprintf(f,"  StartRatio:%d  EndRatio:%d ",d->startRatio,d->endRatio);
		fprintf(f,"  Start color:(%d,%d,%d,%d) ",d->startColor.red,d->startColor.green,d->startColor.blue,d->startColor.alpha);	
		fprintf(f,"  End color:(%d,%d,%d,%d) ",d->endColor.red,d->endColor.green,d->endColor.blue,d->endColor.alpha);	
	}
}

void Swf_decoder::dumpString(FILE *f , STRING *s) {
	fprintf(f,"  String:");
	fprintWstring(f,s->getString());
	//for (int i=0; i< strlen(st); i++) fprintf(f," %02x",(char)st[i]); 
	fprintf(f,"\n");
}

void Swf_decoder::dumpFillStyle(FILE *f, FILLSTYLE *r) {
	int type = r->getFillStyleType();
	fprintf(f,"     fillStyleType:%x\n",type);
	if (type ==0) fprintf(f,"     Color : (%d,%d,%d,%d)\n",r->getColor()->red,r->getColor()->green,r->getColor()->blue,r->getColor()->alpha);
	if (type == 0x10 || type == 0x12) {
		dumpMatrix(f,r->getGradientMatrix());
		dumpGradient(f,r->getGradient());
	}
	if (type == 0x40 || type == 0x41 || type == 0x42 || type == 0x43){
		fprintf(f,"     bitmapId: %d\n",r->getBitmapId());
		dumpMatrix(f,r->getBitmapMatrix());
	}
}

void Swf_decoder::dumpMorphFillStyle(FILE *f, MORPHFILLSTYLE *r) {
	int type = r->getFillStyleType();
	fprintf(f,"     MorphFillStyleType:%x\n",type);
	if (type ==0)	{
		 fprintf(f,"     Start Color : (%d,%d,%d,%d)\n",r->getStartColor()->red,r->getStartColor()->green,r->getStartColor()->blue,r->getStartColor()->alpha);
		 fprintf(f,"     End Color : (%d,%d,%d,%d)\n",r->getEndColor()->red,r->getEndColor()->green,r->getEndColor()->blue,r->getEndColor()->alpha);
	}
	if (type == 0x10 || type == 0x12) {
		dumpMatrix(f,r->getStartGradientMatrix());
		dumpMatrix(f,r->getEndGradientMatrix());
		dumpMorphGradient(f,r->getGradient());
	}
	if (type == 0x40 || type == 0x41 || type == 0x42 || type == 0x43){
		fprintf(f,"     bitmapId: %d\n",r->getBitmapId());
		dumpMatrix(f,r->getStartBitmapMatrix());
		dumpMatrix(f,r->getEndBitmapMatrix());
	}
}

void Swf_decoder::dumpLineStyle(FILE *f, LINESTYLE *r) {
	fprintf(f,"  LINESTYLE: Width:%d  color:(%d,%d,%d,%d) \n",r->width,r->color.red,r->color.green,r->color.blue,r->color.alpha);
}

void Swf_decoder::dumpMorphLineStyle(FILE *f, MORPHLINESTYLE *r) {
	fprintf(f,"  LINESTYLE: StartWidth:%d  Start color:(%d,%d,%d,%d) \n",r->startWidth,r->startColor.red,r->startColor.green,r->startColor.blue,r->startColor.alpha);
	fprintf(f,"  LINESTYLE: EndWidth:%d    End color:(%d,%d,%d,%d) \n",r->endWidth,r->endColor.red,r->endColor.green,r->endColor.blue,r->endColor.alpha);
}

void Swf_decoder::dumpRecord(FILE *f, ShapeRecord *r) {
	UI8 type = r->getTypeFlag();
	switch (type) {
		case EndShapeRecordType:	
			fprintf(f,"  EndShapeRecordType\n");
			break;
		case StyleChangeRecordType:	{
			StyleChangeRecord *s = (StyleChangeRecord*) r;
			fprintf(f,"  StyleChangeRecordType\n");
			fprintf(f,"   StateNewStyles:%d\n",s->getStateNewStyles());	
			fprintf(f,"   StateLineStyle:%d\n",s->getStateLineStyle());
			fprintf(f,"   StateFillStyle1:%d\n",s->getStateFillStyle1());
			fprintf(f,"   StateFillStyle0:%d\n",s->getStateFillStyle0());
			fprintf(f,"   StateMoveTo:%d\n",s->getStateMoveTo());
			if (s->getStateMoveTo() == 1) {
				fprintf(f,"   MoveDeltaX:%d\n",s->getMoveDeltaX());
				fprintf(f,"   MoveDeltaY:%d\n",s->getMoveDeltaY());
			}
			if (s->getStateFillStyle0() == 1) {
				fprintf(f,"   FillStyle0:%d\n",s->getFillStyle0());
			}
			if (s->getStateFillStyle1() == 1) {
				fprintf(f,"   FillStyle1:%d\n",s->getFillStyle1());
			}
			if (s->getStateLineStyle() == 1) {
				fprintf(f,"   LineStyle:%d\n",s->getLineStyle());
			}
			if (s->getStateNewStyles() == 1) {
				fprintf(f,"   StateNewStyles\n");
				fprintf(f,"    NumFillStyles:%d\n",s->getNumFillStyles());
				for (int i=0; i< s->getNumFillStyles(); i++) 
					dumpFillStyle(f,&s->getFillStyles()[i]);
				fprintf(f,"    NumLineStyles:%d\n",s->getNumLineStyles());
				for (int i=0; i< s->getNumLineStyles(); i++) 
					dumpLineStyle(f,&s->getLineStyles()[i]);
					
				fprintf(f,"    NumFillBits:%d\n",s->getNumFillBits());
				fprintf(f,"    NumLineBits:%d\n",s->getNumLineBits());
			}
			break; }
		case StraightEdgeRecordType: {	
			fprintf(f,"  StraightEdgeRecordType\n");
			StraightEdgeRecord *s = (StraightEdgeRecord*) r;
			fprintf(f,"   StraightFlag:%d\n",s->getStraightFlag());
			fprintf(f,"   GeneralLineFlag:%d\n",s->getGeneralLineFlag());
			if (s->getGeneralLineFlag() == 1) {
				fprintf(f,"   DeltaX:%d DeltaY:%d\n",s->getDeltaX(),s->getDeltaY());
			} else {
				fprintf(f,"   VertLineFlag:%d\n",s->getVertLineFlag());
				fprintf(f,"   DeltaVert:%d \n",s->getDeltaVert());
			}
			break; }
		case CurvedEdgeRecordType:	
			fprintf(f,"  CurvedEdgeRecordType\n");
			CurvedEdgeRecord *s = (CurvedEdgeRecord*) r;
			fprintf(f,"   controlDeltaX:%d, controlDeltaY:%d, anchorDeltaX:%d, anchorDeltaY:%d\n",s->getControlDeltaX(),s->getControlDeltaY() ,s->getAnchorDeltaX(),s->getAnchorDeltaY());
			break;
	}
}

void Swf_decoder::dumpShapeWithStyle(FILE *f, ShapeWithStyle *r) {
	fprintf(f," NumFillStyles: %d\n",r->getNumFillStyles());
	for (int i=0; i< r->getNumFillStyles(); i++)
		dumpFillStyle(f,&r->getFillStyles()[i]);
	fprintf(f," NumLineStyles: %d\n",r->getNumLineStyles());
	for (int i=0; i < r->getNumLineStyles(); i++)
		dumpLineStyle(f,&r->getLineStyles()[i]);
	fprintf(f," numFillBits: %d\n",r->getNumFillBits());
	fprintf(f," numLineBits: %d\n",r->getNumLineBits());
	
	list<ShapeRecord*>::iterator it;
	for (it = r->getShapeRecords()->begin(); it != r->getShapeRecords()->end(); it++) {
		dumpRecord(f,*it);
	}
}

void Swf_decoder::dumpShape(FILE *f, Shape *r) {
	fprintf(f,"  numFillBits: %d\n",r->getNumFillBits());
	fprintf(f,"  numLineBits: %d\n",r->getNumLineBits());
	
	list<ShapeRecord*>::iterator it;
	for (it = r->getShapeRecords()->begin(); it != r->getShapeRecords()->end(); it++) {
		dumpRecord(f,*it);
	}
}

void Swf_decoder::dumpDefineShapeTag(FILE *f, DefineShapeTag *r) {
	fprintf(f,"** TAG defineShape\n");
	fprintf(f," ShapeID:%d\n",r->getShapeId());
	dumpRect(f,r->getShapeBounds());
	dumpShapeWithStyle(f,r->getShapeWithStyle());
	fprintf(f,"\n");
}
void Swf_decoder::dumpDefineShapeTag2(FILE *f, DefineShapeTag *r) {
	fprintf(f,"** TAG defineShape2\n");
	fprintf(f," ShapeID:%d\n",r->getShapeId());
	dumpRect(f,r->getShapeBounds());
	dumpShapeWithStyle(f,r->getShapeWithStyle());
	fprintf(f,"\n");
}

void Swf_decoder::dumpCxForm(FILE *f, CXFORM *s) {
	fprintf(f," CxForm section\n");
	fprintf(f,"  hasAddTerms:%d\n",s->hasAddTerms);
	fprintf(f,"  hasMultTerms:%d\n",s->hasMultTerms);	
	fprintf(f,"  redMultTerm:%d\n",s->redMultTerm);
	fprintf(f,"  greenMultTerm:%d\n",s->greenMultTerm);
	fprintf(f,"  blueMultTerm:%d\n",s->blueMultTerm);
	fprintf(f,"  redAddTerm:%d\n",s->redAddTerm);
	fprintf(f,"  greenAddTerm:%d\n",s->greenAddTerm);
	fprintf(f,"  blueAddTerm:%d\n",s->blueAddTerm);
	
}
void Swf_decoder::dumpCxFormWithAlpha(FILE *f, CXFORMWITHALPHA *s) {
	fprintf(f," CxFormWithAlpha section\n");
	fprintf(f,"  hasAddTerms:%d\n",s->hasAddTerms);
	fprintf(f,"  hasMultTerms:%d\n",s->hasMultTerms);
	if (s->hasMultTerms) {	
		fprintf(f,"  redMultTerm:%d\n",s->redMultTerm);
		fprintf(f,"  greenMultTerm:%d\n",s->greenMultTerm);
		fprintf(f,"  blueMultTerm:%d\n",s->blueMultTerm);
		fprintf(f,"  alphaMultTerm:%d\n",s->alphaMultTerm);
	}
	if (s->hasAddTerms) {
		fprintf(f,"  redAddTerm:%d\n",s->redAddTerm);
		fprintf(f,"  greenAddTerm:%d\n",s->greenAddTerm);
		fprintf(f,"  blueAddTerm:%d\n",s->blueAddTerm);
		fprintf(f,"  alphaAddTerm:%d\n",s->alphaAddTerm);
	}
}

void Swf_decoder::dumpPlaceObject(FILE *f, PlaceObjectTag *r) {
	fprintf(f,"  CharacterId:%d\n",r->getCharacterId());
	fprintf(f,"** TAG PlaceObject\n");
	fprintf(f," Depth:%d\n",r->getDepth());
	dumpMatrix(f,r->getMatrix());
	dumpCxForm(f,r->getColorTransform());
}

void Swf_decoder::dumpPlaceObject2(FILE *f, PlaceObject2Tag *r) {
	fprintf(f,"** TAG PlaceObject2\n");
	fprintf(f," PlaceFlagHasClipActions:%d\n",r->getPlaceFlagHasClipActions());
	fprintf(f," PlaceFlagHasClipDepth:%d\n",r->getPlaceFlagHasClipDepth());
	fprintf(f," PlaceFlagHasName:%d\n",r->getPlaceFlagHasName());
	fprintf(f," PlaceFlagHasRatio:%d\n",r->getPlaceFlagHasRatio());
	fprintf(f," PlaceFlagHasColorTransform:%d\n",r->getPlaceFlagHasColorTransform());
	fprintf(f," PlaceFlagHasMatrix:%d\n",r->getPlaceFlagHasMatrix());
	fprintf(f," PlaceFlagHasCharacter:%d\n",r->getPlaceFlagHasCharacter());
	fprintf(f," PlaceFlagMove:%d\n",r->getPlaceFlagMove());
	fprintf(f," Depth:%d\n",r->getDepth());
	if (r->getPlaceFlagHasMatrix()) dumpMatrix(f,r->getMatrix());
	if (r->getPlaceFlagHasName()) dumpString(f,r->getName());
	if (r->getPlaceFlagHasCharacter()) fprintf(f,"  CharacterId:%d\n",r->getCharacterId());
	if (r->getPlaceFlagHasColorTransform()) dumpCxFormWithAlpha(f,r->getColorTransform());
	if (r->getPlaceFlagHasRatio()) fprintf(f," Ratio:%d\n",r->getRatio());
	if (r->getPlaceFlagHasClipDepth()) fprintf(f," ClipDepth:%d\n",r->getClipDepth());
	if (r->getPlaceFlagHasClipActions()) {
		fprintf(f," CLIP ACTIONS SECTION\n");
		dumpClipActions(f,r->getClipActions());
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineFont(FILE *f, DefineFontTag *r) {
	fprintf(f,"** TAG DefineFont\n");
	fprintf(f," FontId:%d\n",r->getFontID());
	fprintf(f," Offsets: ");
	int n = r->getOffsetTable()[0];
	for (int i= 1; i< n ; i++) fprintf(f,"   %d ",r->getOffsetTable()[i]); fprintf(f,"\n");
	for (int i= 0; i< n ; i++) {
		fprintf(f," SHAPE : %d\n",i);
		dumpShape(f,&r->getGlyphShapeTable()[i]);
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineFontInfo(FILE *f, DefineFontInfoTag *r) {
	fprintf(f,"** TAG DefineFontInfo\n");
	fprintf(f," FontId:%d\n",r->getFontID());
	fprintf(f," FontName:%s\n",r->getFontName());
	fprintf(f," FontFlagsSmallText:%d\n",r->getFontFlagsSmallText());
	fprintf(f," FontFlagsShiftJIS:%d\n",r->getFontFlagsShiftJIS());
	fprintf(f," FontFlagsANSI:%d\n",r->getFontFlagsANSI());
	fprintf(f," FontFlagsItalic:%d\n",r->getFontFlagsItalic());
	fprintf(f," FontFlagsBold:%d\n",r->getFontFlagsBold());
	fprintf(f," FontFlagsWideCodes:%d\n",r->getFontFlagsWideCodes());
	fprintf(f," Code Table: ");
	for (int i=0; i< r->getCodeTableLen(); i++) {
		fprintf(f," %d ",r->getCodeTable()[i]);
	}
	fprintf(f,"\n\n");
}

void Swf_decoder::dumpDefineFontInfo2(FILE *f, DefineFontInfo2Tag *r) {
	fprintf(f,"** TAG DefineFontInfo2\n");
	fprintf(f," FontId:%d\n",r->getFontID());
	fprintf(f," FontName:%s\n",r->getFontName());
	fprintf(f," FontFlagsSmallText:%d\n",r->getFontFlagsSmallText());
	fprintf(f," FontFlagsShiftJIS:%d\n",r->getFontFlagsShiftJIS());
	fprintf(f," FontFlagsANSI:%d\n",r->getFontFlagsANSI());
	fprintf(f," FontFlagsItalic:%d\n",r->getFontFlagsItalic());
	fprintf(f," FontFlagsBold:%d\n",r->getFontFlagsBold());
	fprintf(f," FontFlagsWideCodes:%d\n",r->getFontFlagsWideCodes());
	fprintf(f," LanguageCode:%d\n",r->getLanguageCode());
	fprintf(f," Code Table: ");
	for (int i=0; i< r->getCodeTableLen(); i++) {
		fprintf(f," %d ",r->getCodeTable()[i]);
	}
	fprintf(f,"\n\n");
}

void Swf_decoder::dumpTextRecord(FILE *f, TEXTRECORD *r) {
	fprintf(f,"  StyleFlagsHasFont:%d\n",r->getStyleFlagsHasFont());
	fprintf(f,"  StyleFlagsHasColor:%d\n",r->getStyleFlagsHasColor());
	fprintf(f,"  StyleFlagsHasYOffset:%d\n",r->getStyleFlagsHasYOffset());
	fprintf(f,"  StyleFlagsHasXOffset:%d\n",r->getStyleFlagsHasXOffset());
	if (r->getStyleFlagsHasFont()) fprintf(f,"  FontID:%d\n",r->getFontID());
	if (r->getStyleFlagsHasColor()) fprintf(f," Color : (%d,%d,%d,%d)\n",r->getTextColor().red,r->getTextColor().green,r->getTextColor().blue,r->getTextColor().alpha);
	if (r->getStyleFlagsHasXOffset()) fprintf(f,"  XOffset:%d\n",r->getXOffset());
	if (r->getStyleFlagsHasYOffset())fprintf(f,"  YOffset:%d\n",r->getYOffset());
	if (r->getStyleFlagsHasFont()) fprintf(f,"  TextHeight:%d\n",r->getTextHeight());
	fprintf(f,"  GlyphCount:%d  ",r->getGlyphCount());
	for (int i=0; i< r->getGlyphCount(); i++) {
		fprintf(f,"(%d,%d)",r->getGlyphEntries()[i].glyphIndex,r->getGlyphEntries()[i].glyphAdvance);
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineText(FILE *f, DefineTextTag *r) {
	fprintf(f,"** TAG DefineText\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	dumpRect(f,r->getTextBounds());
	dumpMatrix(f,r->getTextMatrix());
	fprintf(f," GlyphBits:%d\n",r->getGlyphBits());
	fprintf(f," AdvanceBits:%d\n",r->getAdvanceBits());
	list<TEXTRECORD*>::iterator it;
	for (it = r->getTextRecords()->begin(); it != r->getTextRecords()->end(); it++) {
		dumpTextRecord(f,*it);
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineSprite(FILE* f,DefineSpriteTag *r) {
	fprintf(f,"** TAG DefineSprite\n");
	fprintf(f," SpriteID:%d\n",r->getSpriteID());
	fprintf(f," FrameCount:%d\n",r->getFrameCount());
	list<BaseTag*>::iterator it;
	for (it = r->getTags()->begin(); it != r->getTags()->end(); it++) {
		BaseTag *p = *it;
		fprintf(f," tag:%d\n",p->getType());
	}

	fprintf(f,"\n");
}

void Swf_decoder::dumpRemoveObject2(FILE *f, RemoveObject2Tag *r) {
	fprintf(f,"** TAG RemoveObject2\n");
	fprintf(f," Depth:%d\n",r->getDepth());
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineBitsLossless(FILE *f,DefineBitsLosslessTag *r) {
	fprintf(f,"** TAG DefineBitsLossless\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	fprintf(f," BitmapFormat:%d\n",r->getBitmapFormat()); 
	fprintf(f," BitmapWidth:%d\n",r->getBitmapWidth()); 
	fprintf(f," BitmapHeight:%d\n",r->getBitmapHeight()); 
	if (r->getBitmapFormat() == 3) fprintf(f," BitmapColorTableSize:%d\n",r->getBitmapColorTableSize());
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineFont2(FILE *f, DefineFont2Tag*r) {
	fprintf(f,"** TAG DefineFont2\n");
	fprintf(f," FontID:%d\n",r->getFontID());
	fprintf(f," FontFlagsHasLayout:%d\n",r->getFontFlagsHasLayout());
	fprintf(f," FontFlagsShiftJIS:%d\n",r->getFontFlagsShiftJIS());
	fprintf(f," FontFlagsSmallText:%d\n",r->getFontFlagsSmallText());
	fprintf(f," FontFlagsANSI:%d\n",r->getFontFlagsANSI());
	fprintf(f," FontFlagsWideOffsets:%d\n",r->getFontFlagsWideOffsets());
	fprintf(f," FontFlagsWideCodes:%d\n",r->getFontFlagsWideCodes());
	fprintf(f," FontFlagsItalic:%d\n",r->getFontFlagsItalic());
	fprintf(f," FontFlagsBold:%d\n",r->getFontFlagsBold());
	fprintf(f," LanguageCode:%d\n",r->getLanguageCode());
	fprintf(f," FontNameLen:%d\n",r->getFontNameLen());
	fprintf(f," FontName:%s\n",r->getFontName());
	fprintf(f," NumGlyphs:%d\n",r->getNumGlyphs());
	fprintf(f," OffsetTable:");
	for (int i=0; i< r->getNumGlyphs(); i++) fprintf(f," %d",r->getOffsetTable()[i]);
	fprintf(f,"\n  CodeTableOffset:%d\n",r->getCodeTableOffset());
	for (int i=0; i< r->getNumGlyphs(); i++)
		dumpShape(f,&(r->getGlyphShapeTable()[i]));
	fprintf(f," codeTable:");
	for (int i=0; i< r->getNumGlyphs(); i++) fprintf(f," %d",r->getCodeTable()[i]);
	fprintf(f,"\n");
	if (r->getFontFlagsHasLayout()) {
		fprintf(f," FontAscent:%d\n",r->getFontAscent());
		fprintf(f," FontDescent:%d\n",r->getFontDescent());
		fprintf(f," FontLeading:%d\n",r->getFontLeading());
		fprintf(f," FontAdvanceTable:");
		for (int i=0; i< r->getNumGlyphs(); i++) fprintf(f," %d",r->getFontAdvanceTable()[i]);
		fprintf(f,"\n fontBoundsTable");
		for (int i=0; i< r->getNumGlyphs(); i++) dumpRect(f,&(r->getFontBoundsTable()[i]));
		fprintf(f," KerningCount:%d\n",r->getKerningCount());
		fprintf(f," KERNINGRECORD\n");
		for (int i=0; i< r->getKerningCount(); i++) {
			KERNINGRECORD *k = &(r->getFontKerningTable()[i]);
			fprintf(f,"   %d %d %d\n",k->fontKerningCode1,k->fontKerningCode2,k->fontKerningAdjustment);
		}
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineEditTextTag(FILE *f, DefineEditTextTag *r) {
	fprintf(f,"** TAG DefineEditText\n");
	fprintf(f," characterID:%d\n",r->getCharacterID());
	RECT re =  r->getBounds();
	dumpRect(f,&re);
	fprintf(f," HasText:%d\n",r->getHasText());
	fprintf(f," WordWrap:%d\n",r->getWordWrap());
	fprintf(f," Multiline:%d\n",r->getMultiline());
	fprintf(f," Password:%d\n",r->getPassword());
	fprintf(f," ReadOnly:%d\n",r->getReadOnly());
	fprintf(f," HasTextColor:%d\n",r->getHasTextColor());
	fprintf(f," HasMaxLength:%d\n",r->getHasMaxLength());
	fprintf(f," HasFont:%d\n",r->getHasFont());
	fprintf(f," AutoSize:%d\n",r->getAutoSize());
	fprintf(f," HasLayout:%d\n",r->getHasLayout());
	fprintf(f," NoSelect:%d\n",r->getNoSelect());
	fprintf(f," Border:%d\n",r->getBorder());
	fprintf(f," HTML:%d\n",r->getHTML());
	fprintf(f," UseOutlines:%d\n",r->getUseOutlines());
	if (r->getHasFont()) {
		fprintf(f," FontID:%d\n",r->getFontID());
		fprintf(f," FontHeight:%d\n",r->getFontHeight());
	}
	if (r->getHasTextColor()) {
		RGBA a = *r->getTextColor();
		fprintf(f," Color : (%d,%d,%d,%d)\n",a.red,a.green,a.blue,a.alpha);
	}
	if (r->getHasMaxLength()) fprintf(f," MaxLength:%d\n",r->getMaxLength());
	if (r->getHasLayout()) {
		fprintf(f," Align:%d\n",r->getAlign());
		fprintf(f," LeftMargin:%d\n",r->getLeftMargin());
		fprintf(f," RightMargin:%d\n",r->getRightMargin());
		fprintf(f," Indent:%d\n",r->getIndent());
		fprintf(f," Leading:%d\n",r->getLeading());
	}
	dumpString(f,r->getVariableName());
	if (r->getHasText()) {
		dumpString(f,r->getInitialText());
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpBUTTONRECORD(FILE *f,BUTTONRECORD *r) {
	fprintf(f,"  buttonStateHitTest:%d\n",r->buttonStateHitTest);
	fprintf(f,"  buttonStateDown:%d\n",r->buttonStateDown);	
	fprintf(f,"  buttonStateOver:%d\n",r->buttonStateOver);
	fprintf(f,"  buttonStateUp:%d\n",r->buttonStateUp);
	fprintf(f,"  characterID:%d\n",r->characterID);
	fprintf(f,"  placeDepth:%d\n",r->placeDepth);
	dumpMatrix(f,&r->placeMatrix);
	fprintf(f,"  CxFormWithAlpha area\n");

}

void Swf_decoder::dumpBUTTONCONDACTION(FILE *f,BUTTONCONDACTION *r) {
	fprintf(f,"  BUTTONCONDACTION\n");
	fprintf(f,"   CondActionSize:%d\n",r->getCondActionSize());
	fprintf(f,"   CondIdleToOverDown:%d\n",r->getCondIdleToOverDown());
	fprintf(f,"   CondOutDownToIdle:%d\n",r->getCondOutDownToIdle());
	fprintf(f,"   CondOutDownToOverDown:%d\n",r->getCondOutDownToOverDown());
	fprintf(f,"   CondOverDownToOutDown:%d\n",r->getCondOverDownToOutDown());
	fprintf(f,"   CondOverDownToOverUp:%d\n",r->getCondOverDownToOverUp());
	fprintf(f,"   CondOverUpToOverDown:%d\n",r->getCondOverUpToOverDown());
	fprintf(f,"   CondOverUpToIdle:%d\n",r->getCondOverUpToIdle());
	fprintf(f,"   CondIdleToOverUp:%d\n",r->getCondIdleToOverUp());
	fprintf(f,"   CondKeyPress:%d\n",r->getCondKeyPress());
	fprintf(f,"   CondOverDownToIdle:%d\n",r->getCondOverDownToIdle());
	dumpActionCode(f,r->getCode());
}

void Swf_decoder::dumpDefineButton(FILE *f,DefineButtonTag *r){
	fprintf(f,"** TAG DefineButton\n");
	fprintf(f," ButtonId:%d\n",r->getButtonId());
	list<BUTTONRECORD*>::iterator it;
	for (it = r->getCharacters()->begin(); it != r->getCharacters()->end(); it++) {
		dumpBUTTONRECORD(f,*it);
	}
	dumpActionCode(f,r->getCode());
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineButton2(FILE *f,DefineButton2Tag *r){
	fprintf(f,"** TAG DefineButton2\n");
	fprintf(f," ButtonId:%d\n",r->getButtonId());
	fprintf(f," TrackAsMenu:%d\n",r->getTrackAsMenu());
	fprintf(f," ActionOffset:%d\n",r->getActionOffset());
	list<BUTTONRECORD*>::iterator it;
	for (it = r->getCharacters()->begin(); it != r->getCharacters()->end(); it++) {
		dumpBUTTONRECORD(f,*it);
	}
	list<BUTTONCONDACTION*>::iterator it2;
	for (it2 = r->getActions()->begin(); it2 != r->getActions()->end(); it2++) {
		dumpBUTTONCONDACTION(f,*it2);
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpFrameLabel(FILE *f, FrameLabelTag *r) {
	fprintf(f,"** TAG FrameLabel\n");
	dumpString(f,r->getName());
	fprintf(f,"\n");
}

void Swf_decoder::dumpExportAssets(FILE *f, ExportAssetsTag *r) {
	fprintf(f,"** TAG ExportAssets\n");
	fprintf(f," Count:%d\n",r->getCount());
	for (int i=0; i< r->getCount(); i++) {
		fprintf(f," Tag:%d\n",r->getTags()[i]);
		dumpString(f,r->getNames()[i]);
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpDoInitAction(FILE *f, DoInitActionTag *r) {
	fprintf(f,"** TAG DoInitAction\n");
	fprintf(f," SpriteID:%d\n",r->getSpriteID());
	dumpActionCode(f,r->getCode());
	fprintf(f,"\n");
}

void Swf_decoder::dumpClipActions(FILE *f, ClipActions *r) {
	for (list<ClipActionRecord*>::iterator it=r->getClipActionRecords()->begin(); it!=r->getClipActionRecords()->end(); it++) {
		ClipActionRecord *ca = *it;
		dumpActionCode(f,ca->getCode());
	}
}

void Swf_decoder::dumpClipEventFlags(FILE *f, ClipEventFlags *r) {
	fprintf(f," ClipEvent\n");
	fprintf(f,"  clipEventKeyUp:%d\n",r->clipEventKeyUp);
	fprintf(f,"  clipEventKeyDown:%d\n",r->clipEventKeyDown);
	fprintf(f,"  clipEventMouseUp:%d\n",r->clipEventMouseUp);
	fprintf(f,"  clipEventMouseDown:%d\n",r->clipEventMouseDown);
	fprintf(f,"  clipEventMouseMove:%d\n",r->clipEventMouseMove);
	fprintf(f,"  clipEventUnload:%d\n",r->clipEventUnload);
	fprintf(f,"  clipEventEnterFrame:%d\n",r->clipEventEnterFrame);
	fprintf(f,"  clipEventLoad:%d\n",r->clipEventLoad);
	fprintf(f,"  clipEventDragOver:%d\n",r->clipEventDragOver);
	fprintf(f,"  clipEventRollOut:%d\n",r->clipEventRollOut);
	fprintf(f,"  clipEventRollOver:%d\n",r->clipEventRollOver);
	fprintf(f,"  clipEventReleaseOutside:%d\n",r->clipEventReleaseOutside);
	fprintf(f,"  clipEventRelease:%d\n",r->clipEventRelease);
	fprintf(f,"  clipEventPress:%d\n",r->clipEventPress);
	fprintf(f,"  clipEventInitialize:%d\n",r->clipEventInitialize);
	fprintf(f,"  clipEventData:%d\n",r->clipEventData);
	if (header.version >= 6) {
		fprintf(f,"  clipEventConstruct:%d\n",r->clipEventConstruct);
		fprintf(f,"  clipEventKeyPress:%d\n",r->clipEventKeyPress);
		fprintf(f,"  clipEventDragOut:%d\n",r->clipEventDragOut);
	}
}

void Swf_decoder::dumpSetTabIndex(FILE* f,SetTabIndexTag *r) {
	fprintf(f,"** TAG SetTabIndex\n");
	fprintf(f," Depth:%d\n",r->getDepth());
	fprintf(f," TabIndex:%d\n",r->getTabIndex());
	fprintf(f,"\n");
}

void Swf_decoder::dumpShowFrame(FILE *f, ShowFrameTag *r) {
	fprintf(f,"** TAG ShowFrame\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineBitsLossless2(FILE *f,DefineBitsLossless2Tag *r) {
	fprintf(f,"** TAG DefineBitsLossless2\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	fprintf(f," BitmapFormat:%d\n",r->getBitmapFormat());
	fprintf(f," BitmapWidth:%d\n",r->getBitmapWidth());
	fprintf(f," BitmapHeight:%d\n",r->getBitmapHeight());
	if (r->getBitmapFormat()==3) fprintf(f," BitmapColorTableSize:%d\n",r->getBitmapColorTableSize());
	fprintf(f," Binary Data.....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineBitsJPEG2(FILE *f, DefineBitsJPEG2Tag *r) {
	fprintf(f,"** TAG DefineBitsJPEG2\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	fprintf(f," JPEG encoding Table binary Data.....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineBits(FILE *f, DefineBitsTag *r) {
	fprintf(f,"** TAG DefineBits\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	fprintf(f," JPEG encoding Table binary Data.....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineBitsJPEG3(FILE *f, DefineBitsJPEG3Tag *r) {
	fprintf(f,"** TAG DefineBitsJPEG3\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	fprintf(f," AlphaDataOffset:%d\n",r->getAlphaDataOffset());
	fprintf(f," JPEG encoding Table binary Data.....\n");
	fprintf(f," JPEG Bitmap alpha Data binary .....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpSoundStreamHead(FILE *f, SoundStreamHeadTag *r) {
	fprintf(f,"** TAG SoundStreamHead\n");
	fprintf(f," PlaybackSoundRate:%d\n",r->getPlaybackSoundRate());
	fprintf(f," PlaybackSoundSize:%d\n",r->getPlaybackSoundSize());
	fprintf(f," PlaybackSoundType:%d\n",r->getPlaybackSoundType());
	fprintf(f," StreamSoundCompression:%d\n",r->getStreamSoundCompression());
	fprintf(f," StreamSoundRate:%d\n",r->getStreamSoundRate());
	fprintf(f," StreamSoundSize:%d\n",r->getStreamSoundSize());
	fprintf(f," StreamSoundType:%d\n",r->getStreamSoundType());
	fprintf(f," StreamSoundSampleCount:%d\n",r->getStreamSoundSampleCount());
	if (r->getStreamSoundCompression() ==2) fprintf(f," LatencySeek:%d\n",r->getLatencySeek());
	fprintf(f,"\n");
}

void Swf_decoder::dumpSoundStreamHead2(FILE *f, SoundStreamHead2Tag *r) {
	fprintf(f,"** TAG SoundStreamHead2\n");
	fprintf(f," PlaybackSoundRate:%d\n",r->getPlaybackSoundRate());
	fprintf(f," PlaybackSoundSize:%d\n",r->getPlaybackSoundSize());
	fprintf(f," PlaybackSoundType:%d\n",r->getPlaybackSoundType());
	fprintf(f," StreamSoundCompression:%d\n",r->getStreamSoundCompression());
	fprintf(f," StreamSoundRate:%d\n",r->getStreamSoundRate());
	fprintf(f," StreamSoundSize:%d\n",r->getStreamSoundSize());
	fprintf(f," StreamSoundType:%d\n",r->getStreamSoundType());
	fprintf(f," StreamSoundSampleCount:%d\n",r->getStreamSoundSampleCount());
	if (r->getStreamSoundCompression() ==2) fprintf(f," LatencySeek:%d\n",r->getLatencySeek());
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineSound(FILE *f,DefineSoundTag *r) {
	fprintf(f,"** TAG DefineSound\n");
	fprintf(f," SoundId:%d\n",r->getSoundId());
	fprintf(f," SoundFormat:%d\n",r->getSoundFormat());
	fprintf(f," SoundRate:%d\n",r->getSoundRate());
	fprintf(f," SoundSize:%d\n",r->getSoundSize());
	fprintf(f," SoundType:%d\n",r->getSoundType());
	fprintf(f," SoundSampleCount:%d\n",r->getSoundSampleCount());
	fprintf(f," Sound data binary....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpSoundInfo(FILE *f, SOUNDINFO *s) {
	fprintf(f," SoundInfo\n");
	fprintf(f,"  SyncStop:%d\n",s->getSyncStop());
	fprintf(f,"  SyncNoMultiple:%d\n",s->getSyncNoMultiple());
	fprintf(f,"  HasEnvelope:%d\n",s->getHasEnvelope());
	fprintf(f,"  HasLoops:%d\n",s->getHasLoops());
	fprintf(f,"  HasOutPoint:%d\n",s->getHasOutPoint());
	fprintf(f,"  HasInPoint:%d\n",s->getHasInPoint());
	if (s->getHasInPoint()) fprintf(f,"  InPoint:%d\n",s->getInPoint());
	if (s->getHasOutPoint()) fprintf(f,"  OutPoint:%d\n",s->getOutPoint());
	if (s->getHasLoops()) fprintf(f,"  LoopCount:%d\n",s->getLoopCount());
	if (s->getHasEnvelope()) {
		fprintf(f,"  EnvPoints:%d\n",s->getEnvPoints());
		for (int i=0; i< s->getEnvPoints(); i++) {
			fprintf(f,"  pos44:%d ",s->getEnvelopeRecords()[i].pos44);
			fprintf(f,"leftLevel:%d ",s->getEnvelopeRecords()[i].leftLevel);
			fprintf(f,"rigthLevel:%d\n",s->getEnvelopeRecords()[i].rightLevel);
		}
	}
}

void Swf_decoder::dumpDefineButtonSound(FILE *f, DefineButtonSoundTag *r) {
	fprintf(f,"** TAG DefineButtonSound\n");
	fprintf(f," ButtonId:%d\n",r->getButtonId());
	fprintf(f," ButtonSoundChar0:%d\n",r->getButtonSoundChar0());
	if (r->getButtonSoundChar0() != 0) {
		dumpSoundInfo(f,r->getButtonSoundInfo0());
	}
	fprintf(f," ButtonSoundChar1:%d\n",r->getButtonSoundChar1());
	if (r->getButtonSoundChar1() != 0) {
		dumpSoundInfo(f,r->getButtonSoundInfo1());
	}
	fprintf(f," ButtonSoundChar2:%d\n",r->getButtonSoundChar2());
	if (r->getButtonSoundChar2() != 0) {
		dumpSoundInfo(f,r->getButtonSoundInfo2());
	}
	fprintf(f," ButtonSoundChar3:%d\n",r->getButtonSoundChar3());
	if (r->getButtonSoundChar3() != 0) {
		dumpSoundInfo(f,r->getButtonSoundInfo3());		
	}	
	fprintf(f,"\n");
}

void Swf_decoder::dumpStartSound(FILE *f, StartSoundTag *r) {
	fprintf(f,"** TAG StartSound\n");
	fprintf(f," SoundId:%d\n",r->getSoundId());
	dumpSoundInfo(f,r->getSoundInfo());
	fprintf(f,"\n");
}

void Swf_decoder::dumpJPEGTables(FILE *f, JPEGTablesTag*r) {
	fprintf(f,"** TAG JPEGTables\n");
	fprintf(f," JPEG table binary....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineBitsTag(FILE *f,DefineBitsTag *r) {
	fprintf(f,"** TAG DefineBitsTag\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	fprintf(f," JPEG table binary....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineMorphShape(FILE *f, DefineMorphShapeTag *r) {
	fprintf(f,"** TAG DefineMorphShape\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	dumpRect(f,r->getStartBounds());
	dumpRect(f,r->getEndBounds());
	fprintf(f," Offset:%d\n",r->getOffset());
	fprintf(f," FillStyleCount:%d\n",r->getMorphFillStyles()->getFillStyleCount());
	for (int i=0; i< r->getMorphFillStyles()->getFillStyleCount(); i++) dumpMorphFillStyle(f,&r->getMorphFillStyles()->getFillStyles()[i]);
	for (int i=0; i< r->getMorphLineStyles()->getLineStyleCount(); i++) dumpMorphLineStyle(f,&r->getMorphLineStyles()->getLineStyles()[i]);
	fprintf(f,"\n START EDGDES\n");
	dumpShape(f,r->getStartEdges());
	fprintf(f,"\n END EDGDES\n");
	dumpShape(f,r->getEndEdges());
	fprintf(f,"\n");
}

void Swf_decoder::dumpSoundStreamBlock(FILE *f, SoundStreamBlockTag* r) {
	fprintf(f,"** TAG SoundStreamBlock\n");
	fprintf(f," Sound binary data....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpProtect(FILE *f , ProtectTag *r) {
	fprintf(f,"** TAG Protect\n");
	if (r->getPassword() != NULL) dumpString(f,r->getPassword());
	fprintf(f,"\n");
}

void Swf_decoder::dumpNameCharacter(FILE *f, NameCharacterTag *r) {
	fprintf(f,"** TAG NameCharacter\n");
	fprintf(f," tagId:%d\n",r->getTagId());
	dumpString(f,r->getName());
	fprintf(f,"\n");
}


void Swf_decoder::dumpRemoveObject(FILE *f, RemoveObjectTag *r) {
	fprintf(f,"** TAG RemoveObject\n");
	fprintf(f," CharacterId:%d\n",r->getCharacterId());
	fprintf(f," Depth:%d\n",r->getDepth());
	fprintf(f,"\n");
}

void Swf_decoder::dumpSetBackgroundColor(FILE *f, SetBackgroundColorTag *r) {
	fprintf(f,"** TAG SetBackgroundColor\n");
	fprintf(f," Background color : (%d,%d,%d)\n",r->getRed(),r->getGreen(),r->getBlue());
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineButtonCxform(FILE *f, DefineButtonCxformTag *r) {
	fprintf(f,"** TAG DefineButtonCxform\n");
	fprintf(f," ButtonId:%d\n",r->getButtonId());
	dumpCxForm(f,r->getButtonColorTransform());
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineText2(FILE *f , DefineText2Tag *r) {
	fprintf(f,"** TAG DefineText2\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	dumpRect(f,r->getTextBounds());
	dumpMatrix(f,r->getTextMatrix());
	fprintf(f," GlyphBits:%d\n",r->getGlyphBits());
	fprintf(f," AdvanceBits:%d\n",r->getAdvanceBits());
	list<TEXTRECORD*>::iterator it;
	for (it = r->getTextRecords()->begin(); it != r->getTextRecords()->end(); it++) {
		dumpTextRecord(f,*it);
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpImportAssets(FILE *f, ImportAssetsTag *r) {
	fprintf(f,"** TAG ImportAssets\n");
	dumpString(f,r->getURL());
	fprintf(f," Names import count:%d\n",r->getCount());
	for (int i=0; i< r->getCount(); i++) {
		fprintf(f,"  CharacterId:%d\n",r->getTags()[i]);
		dumpString(f,&r->getNames()[i]);
	}
	fprintf(f,"\n");
}

void Swf_decoder::dumpEnableDebugger(FILE *f, EnableDebuggerTag *r) {
	fprintf(f,"** TAG EnableDebugger\n");
	dumpString(f,r->getPassword());
	fprintf(f,"\n");
}

void Swf_decoder::dumpEnableDebugger2(FILE *f, EnableDebugger2Tag *r) {
	fprintf(f,"** TAG EnableDebugger2\n");
	dumpString(f,r->getPassword());
	fprintf(f,"\n");
}

void Swf_decoder::dumpDefineVideoStream(FILE *f, DefineVideoStreamTag *r) {
	fprintf(f,"** TAG DefineVideoStream\n");
	fprintf(f," CharacterID:%d\n",r->getCharacterID());
	fprintf(f," NumFrames:%d\n",r->getNumFrames());
	fprintf(f," Width:%d\n",r->getWidth());
	fprintf(f," Height:%d\n",r->getHeight());
	fprintf(f," VideoFlagsDeblocking:%d\n",r->getVideoFlagsDeblocking());
	fprintf(f," VideoFlagsSmoothing:%d\n",r->getVideoFlagsSmoothing());
	fprintf(f," CodecID:%d\n",r->getCodecID());
	fprintf(f,"\n");
}

void Swf_decoder::dumpVideoFrame(FILE *f , VideoFrameTag *r) {
	fprintf(f,"** TAG VideoFrame\n");
	fprintf(f," StreamID:%d\n",r->getStreamID());
	fprintf(f," FrameNum:%d\n",r->getFrameNum());
	fprintf(f," Video binary data.....\n");
	fprintf(f,"\n");
}

void Swf_decoder::dumpScriptLimits(FILE *f, ScriptLimitsTag *r) {
	fprintf(f,"** TAG ScriptLimits\n");
	fprintf(f," MaxRecursionDepth:%d\n",r->getMaxRecursionDepth());
	fprintf(f," ScriptTimeoutSeconds:%d\n",r->getScriptTimeoutSeconds());
	fprintf(f,"\n");
}

void Swf_decoder::dumpActionCode(FILE *f, UI8* code) {
	bool cont = true;
	int i=0; 
	while (cont) {
		UI8 a = code[i]; i++;
		UI16 dataLen = 0;
		if ( (a>>7) != 0) {
			dataLen = (UI16) code[i] | ((UI16) code[i+1] << 8);
			i += dataLen+2;
		} 
		cont = a != 0;
		fprintf(f,"  Code:%x  Size:%d \n",a,dataLen);
	}
}
