/**
Macromedia Flash. Function type implementation.
Copyright (C) 2005  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#include "swf_function.hh"

/**
 * RegisterSet class implementation.
 **/
RegisterSet::RegisterSet(int registerCount) {
	registers = new ValueObject*[registerCount];
	for (int i=0; i< registerCount;i++) registers[i] = NULL;
	this->registerCount = registerCount;
	isGlobal = false;
}

RegisterSet::~RegisterSet() {
	//puts("~RegisterSet");
	for (int i=0; i< registerCount;i++) {
		//if (registers[i] != NULL ) {
		//	ValueObject * v = registers[i];
		//	if (v->getObjectType() != ScriptObjectType) delete v;
		//}
		registers[i] = NULL;
		}
	delete []registers;
}

/**
 * Set a register instance to be the global register set.
 * registers won't be deallocated when function is deleted
 **/
void RegisterSet::setGlobal(bool p) { isGlobal = p; }
bool RegisterSet::getIsGlobal() { return isGlobal; }

/**
 * Clone a register set
 *
RegisterSet *RegisterSet::clone() {
	RegisterSet *ret = new RegisterSet(*this);
	//for (int i=0; i< this->registerCount; i++) {
	//	ret->setRegister(i, getRegister(i));
	//}
	ret->setClone(true);
	return ret;
}*/

void RegisterSet::setRegister(int index, ValueObject *p) { 
	if (p!=NULL) registers[index] = p->clone(); else registers[index]= NULL;
}

ValueObject* RegisterSet::getRegister(int index) { 
	return (registers[index] == NULL) ?  NULL : registers[index]->clone(); 
}

ValueObject* RegisterSet::getRegisterNoClone(int index) { 
	return (registers[index] == NULL) ?  NULL : registers[index]; 
}

ValueObject**RegisterSet::getRegisterRef(int index) { 
	return &registers[index]; 
}

int RegisterSet::getRegisterCount() { 
	return registerCount; 
}

/**
 * Function class implementation.
 **/
Function::Function() : ScriptObject(L"function") {
	code = NULL;
	params = NULL;
	isFunction2 = false;
	paramRegisters = NULL;
	isNative = false;
	registers = NULL;
}

Function::~Function() {
	if (params != NULL) delete []params;
	if (paramRegisters != NULL) delete[]paramRegisters;
	if (registers != NULL && !registers->getIsGlobal()) delete registers;
}

wstring Function::getFunctionName() { return functionName; };
int     Function::getNumParams() { return numParams; }
int     Function::getNumPassedParams() { return numPassedParams; }
wstring Function::getParam(int n) { return n< numParams ? params[n] : NULL; }
void	Function::setFunctionName(wstring  p) { functionName = p; };
void	Function::setNumParams(int p) { numParams= p; }
void	Function::setNumPassedParams(int p) { numPassedParams= p; }
void	Function::setParams(wstring *p) { if (params!=NULL) delete params; params = p; }
uint8_t*Function::getCode() { return code; }
void	Function::setCode(uint8_t *p) { code = p; }
bool	Function::getPreloadParentFlag() { return preloadParentFlag;}
bool	Function::getPreloadRootFlag() { return preloadRootFlag;}
bool	Function::getSuppressSuperFlag() { return suppressSuperFlag;}
bool	Function::getPreloadSuperFlag() { return preloadSuperFlag;}
bool	Function::getSuppressArgumentsFlag() { return suppressArgumentsFlag;}
bool	Function::getPreloadArgumentsFlag() { return preloadArgumentsFlag;}
bool	Function::getSuppressThisFlag() { return suppressThisFlag;}
bool	Function::getPreloadThisFlag() { return preloadThisFlag;}
bool	Function::getPreloadGlobalFlag() { return preloadGlobalFlag;}
void	Function::setPreloadParentFlag(bool p) { preloadParentFlag =p;}
void	Function::setPreloadRootFlag(bool p) { preloadRootFlag =p;}
void	Function::setSuppressSuperFlag(bool p) { suppressSuperFlag =p;}
void	Function::setPreloadSuperFlag(bool p) { preloadSuperFlag =p;}
void	Function::setSuppressArgumentsFlag(bool p) { suppressArgumentsFlag =p;}
void	Function::setPreloadArgumentsFlag(bool p) { preloadArgumentsFlag =p;}
void	Function::setSuppressThisFlag(bool p) { suppressThisFlag =p;}
void	Function::setPreloadThisFlag(bool p) { preloadThisFlag =p;}
void	Function::setPreloadGlobalFlag(bool p) { preloadGlobalFlag =p;}
void	Function::setIsFunction2(bool p) { isFunction2 = true; }
bool	Function::getIsFunction2() { return isFunction2; }
void	Function::setRegisterCount(int p) { registerCount = p; }
int		Function::getRegisterCount() { return registerCount; }
void	Function::setParamRegisters(uint8_t* p) { paramRegisters = p;} 
uint8_t*Function::getParamRegisters() { return paramRegisters;}
void	Function::setCodeSize(int p) { codeSize = p; }
int		Function::getCodeSize() { return codeSize; }
void	Function::setRegisters(RegisterSet *p) {registers=p; }
RegisterSet *Function::getRegisters() { return registers; }
void	Function::setIsNative(bool p) { isNative = p; }
bool	Function::getIsNative() { return isNative; }
void	Function::setFunctionIndex(int idx) { functionIndex = idx; }
int		Function::getFunctionIndex() { return functionIndex; }
void 	Function::setObjectInstance(ValueObject *p) { objectInstance = p; }
ValueObject *Function::getObjectInstance() { return objectInstance; }
void 	Function::setClassName(wstring s) { className=s; }
wstring Function::getClassName() { return className; }
void	Function::setConstantPoolKey(wstring p) { constantPoolKey = p; }
wstring	Function::getConstantPoolKey() { return constantPoolKey; }
void	Function::setInlined(bool p) { isInlined = p; }
bool	Function::getIsInlined() { return isInlined; }
void	Function::setTargetPath(wstring s) { currentTargetPath = s; }
wstring	Function::getTargetPath() { return currentTargetPath; }

