/**
Macromedia Flash. Function type definition.
Copyright (C) 2005  Christophe Choquet (cchoquet@users.sourceforge.net)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
**/
#ifndef _FUNCTION_HH
#define _FUNCTION_HH
#include "common.hh"
#include "object_types.hh"
#include <iostream>
#include <map>

/**
 * A register set defines the register variables passed when calling a function
 **/
class RegisterSet {
public:
	RegisterSet(int registerCount);
	~RegisterSet();
	void setRegister(int index, ValueObject *p);
	ValueObject*getRegister(int index);
	ValueObject*getRegisterNoClone(int index);
	ValueObject**getRegisterRef(int index);
	int getRegisterCount();
//	RegisterSet *clone();
	void	setGlobal(bool p);
	bool	getIsGlobal();
private:
	ValueObject **registers;
	int		registerCount;
	bool	isGlobal;
};


/**
 * Function definition. 
 * This is static definition: number of parameters, calling rules...
 * It has no execution time values.
 **/
class Function : public ScriptObject {
public:
	Function();
	~Function();
	wstring getFunctionName();
	int     getNumParams();
	int     getNumPassedParams();
	wstring getParam(int n);
	void	setFunctionName(wstring  p);
	void	setNumParams(int p);
	void	setNumPassedParams(int p);
	void	setParams(wstring *p);
	uint8_t*getCode();
	void	setCode(uint8_t *p);
	bool	getPreloadParentFlag();
	bool	getPreloadRootFlag();
	bool	getSuppressSuperFlag();
	bool	getPreloadSuperFlag();
	bool	getSuppressArgumentsFlag();
	bool	getPreloadArgumentsFlag();
	bool	getSuppressThisFlag();
	bool	getPreloadThisFlag();
	bool	getPreloadGlobalFlag();
	void	setPreloadParentFlag(bool p);
	void	setPreloadRootFlag(bool p);
	void	setSuppressSuperFlag(bool p);
	void	setPreloadSuperFlag(bool p);
	void	setSuppressArgumentsFlag(bool p);
	void	setPreloadArgumentsFlag(bool p);
	void	setSuppressThisFlag(bool p);
	void	setPreloadThisFlag(bool p);
	void	setPreloadGlobalFlag(bool p);
	void	setIsFunction2(bool p);
	bool	getIsFunction2();
	void	setRegisterCount(int p);
	int		getRegisterCount();
	void	setParamRegisters(uint8_t* p);
	uint8_t*getParamRegisters();
	void	setCodeSize(int p);
	int		getCodeSize();
	void	setRegisters(RegisterSet *p);
	RegisterSet *getRegisters();
	void	setIsNative(bool p);
	bool	getIsNative();
	void	setFunctionIndex(int idx);
	int		getFunctionIndex();
	void 	setObjectInstance(ValueObject *p);
	ValueObject *getObjectInstance();
	void 	setClassName(wstring s);
	wstring getClassName();
	void	setConstantPoolKey(wstring p);
	wstring	getConstantPoolKey();
	void	setInlined(bool p);
	bool	getIsInlined();
	void	setTargetPath(wstring s);
	wstring	getTargetPath();
private:
	wstring functionName;
	int     numParams;
	int     numPassedParams;
	wstring *params;
	uint8_t	*paramRegisters;	
	uint8_t*code;
	int 	codeSize;
	bool	isFunction2;
	bool	isNative;
	bool	preloadParentFlag;
	bool	preloadRootFlag;
	bool	suppressSuperFlag;
	bool	preloadSuperFlag;
	bool	suppressArgumentsFlag;
	bool	preloadArgumentsFlag;
	bool	suppressThisFlag;
	bool	preloadThisFlag;
	bool	preloadGlobalFlag;
	int 	registerCount;
	RegisterSet *registers;
	int		functionIndex;
	ValueObject *objectInstance;// object instance if needed
	wstring	constantPoolKey;
	bool	isInlined;
	wstring	currentTargetPath;	// in case of inlined functions
};

#endif
